# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import touchtechnology.common.db.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('competition', '0006_sportingpulse_import_fields'),
    ]

    operations = [
        migrations.CreateModel(
            name='MatchVideo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField(max_length=255)),
                ('approved_by', touchtechnology.common.db.models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('match', touchtechnology.common.db.models.ForeignKey(related_name='videos', to='competition.Match')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.forms import BooleanField as BooleanChoiceField
from django.utils.translation import ugettext_lazy as _
from modelforms.forms import ModelForm
from touchtechnology.common.forms.mixins import UserMixin

UserModel = get_user_model()


class UserEditForm(UserMixin, ModelForm):
    is_superuser = BooleanChoiceField(label=_("God mode"),
                                      help_text=_("Should this user be all "
                                                  "knowing and all powerful?"))

    class Meta:
        model = UserModel
        fields = '__all__'
        help_texts = {
            'is_active': _("A disabled account cannot be used to "
                           "login to the site."),
            'is_staff': _("Content editor's have access to this "
                          "administration interface."),
        }
        labels = {
            'is_active': _("Enabled"),
            'is_staff': _("Content editor"),
        }


class GroupEditForm(UserMixin, ModelForm):

    def __init__(self, *args, **kwargs):
        super(GroupEditForm, self).__init__(*args, **kwargs)
        # The django.contrib applications don't benefit from the
        # touchtechnology.common database fields, so we need to tweak this
        # field queryset the old fashioned way to make sure we're doing a join.
        self.fields['permissions'].queryset = \
            self.fields['permissions'].queryset.select_related()

    class Meta:
        model = Group
        fields = (
            'name',
            'permissions',
        )
        help_texts = {
            'permissions': _("Members of this group will have "
                             "these permissions."),
        }



import unittest

from django.test.utils import override_settings
from example_app.models import TestDateTimeField
from example_app.tests.factories import TestDateTimeFieldFactory
from test_plus import TestCase as BTC
from touchtechnology.common.tests import factories


@override_settings(ROOT_URLCONF='example_app.urls')
class TestCase(BTC):

    def setUp(self):
        self.staff = factories.UserFactory.create(
            is_staff=True, is_superuser=True)
        self.regular = factories.UserFactory.create()


class GenericListTests(TestCase):

    def test_vanilla(self):
        self.assertGoodView('generic:vanilla:list')
        self.assertResponseContains('<title>test date time field</title>')
        self.assertResponseContains('<h1>test date time fields</h1>')

    def test_permissions_403(self):
        "Ensure unprivileged user is denied, superuser may view"
        self.assertLoginRequired('generic:permissions:list')

        with self.login(self.regular):
            self.get('generic:permissions:list')
            self.response_403()

        with self.login(self.staff):
            self.assertGoodView('generic:permissions:list')


class GenericDetailTests(TestCase):

    def setUp(self):
        super(GenericDetailTests, self).setUp()
        TestDateTimeFieldFactory.reset_sequence()
        TestDateTimeFieldFactory.create_batch(20)
        self.instance = TestDateTimeField.objects.earliest('pk')

    def test_vanilla(self):
        self.assertGoodView('generic:vanilla:detail', self.instance.pk)
        self.assertResponseContains('<title>test date time field</title>')
        self.assertResponseContains('<h1>test date time fields</h1>')
        self.assertResponseContains('<p>{}</p>'.format(self.instance.datetime))

    @unittest.skip("permissions_required not implemented for generic_detail")
    def test_permissions_403(self):
        self.get('generic:permissions:detail', self.instance.pk)
        self.response_403()


class GenericEditTests(TestCase):

    def test_vanilla(self):
        self.assertGoodView('generic:vanilla:add')
        self.assertResponseContains('<title>Edit test date time field</title>')

        data = {
            'datetime_0': '1',
            'datetime_1': '1',
            'datetime_2': '2015',
            'datetime_3': '9',
            'datetime_4': '0',
            'datetime_5': 'UTC',
        }
        res = self.post('generic:vanilla:add', data=data)
        self.assertEqual(1, TestDateTimeField.objects.count())
        self.response_302()
        self.assertRedirects(res, self.reverse('generic:vanilla:list'))

        self.assertGoodView('generic:vanilla:list')
        self.assertResponseContains('<li>2015-01-01 09:00:00+00:00</li>')

    def test_permissions_403(self):
        "Ensure unprivileged user is denied, superuser may view and edit"
        self.assertLoginRequired('generic:permissions:add')

        data = {
            'datetime_0': '1',
            'datetime_1': '1',
            'datetime_2': '2015',
            'datetime_3': '9',
            'datetime_4': '0',
            'datetime_5': 'UTC',
        }

        with self.login(self.regular):
            self.get('generic:permissions:add')
            self.response_403()
            self.post('generic:permissions:add', data=data)
            self.response_403()

        with self.login(self.staff):
            self.assertGoodView('generic:permissions:add')
            self.post('generic:permissions:add', data=data)
            self.response_302()


class GenericDeleteTests(TestCase):

    def setUp(self):
        super(GenericDeleteTests, self).setUp()
        self.object = TestDateTimeFieldFactory.create()

    def test_vanilla(self):
        self.post('generic:vanilla:delete', self.object.pk)
        self.response_302()

    def test_permissions_403(self):
        "Ensure unprivileged user is denied, superuser may delete"
        self.post('generic:permissions:delete', self.object.pk)
        self.response_302()  # Anonymous users will be redirected to login

        with self.login(self.regular):
            self.post('generic:permissions:delete', self.object.pk)
            self.response_403()

        with self.login(self.staff):
            self.post('generic:permissions:delete', self.object.pk)
            self.response_302()


class GenericPermissionsTests(TestCase):

    def setUp(self):
        super(GenericPermissionsTests, self).setUp()
        self.object = TestDateTimeFieldFactory.create()

    def test_vanilla(self):
        self.get('generic:vanilla:perms', self.object.pk)
        self.response_403()

        with self.login(self.regular):
            self.get('generic:vanilla:perms', self.object.pk)
            self.response_403()

        with self.login(self.staff):
            self.assertGoodView('generic:vanilla:perms', self.object.pk)
            self.response_200()
            self.assertResponseContains("""
                <p>
                    Control who <em>can change test date time field</em>
                    for the test date time field
                    <strong>{}</strong>.
                </p>
            """.format(self.object))


class GenericEditMultipleTests(TestCase):

    def test_vanilla(self):
        self.assertGoodView('generic:vanilla:edit')
        self.assertResponseContains(
            '<title>Edit multiple test date time fields</title>')

        data = {
            # management form hidden fields
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 0,
            'form-MIN_NUM_FORMS': 0,
            'form-MAX_NUM_FORMS': 1000,

            'form-0-datetime_0': '1',
            'form-0-datetime_1': '1',
            'form-0-datetime_2': '2015',
            'form-0-datetime_3': '9',
            'form-0-datetime_4': '0',
            'form-0-datetime_5': 'UTC',

            'form-1-datetime_0': '3',
            'form-1-datetime_1': '3',
            'form-1-datetime_2': '2016',
            'form-1-datetime_3': '9',
            'form-1-datetime_4': '0',
            'form-1-datetime_5': 'UTC',
        }
        res = self.post('generic:vanilla:edit', data=data)
        self.assertEqual(2, TestDateTimeField.objects.count())
        self.response_302()
        self.assertRedirects(res, self.reverse('generic:vanilla:list'))

        self.assertGoodView('generic:vanilla:list')
        self.assertResponseContains(
            '<ul>'
            '<li>2015-01-01 09:00:00+00:00</li>'
            '<li>2016-03-03 09:00:00+00:00</li>'
            '</ul>')

    def test_permissions_403(self):
        "Ensure unprivileged user is denied, superuser may view and edit"
        self.assertLoginRequired('generic:permissions:edit')

        data = {
            # management form hidden fields
            'form-TOTAL_FORMS': 2,
            'form-INITIAL_FORMS': 0,
            'form-MIN_NUM_FORMS': 0,
            'form-MAX_NUM_FORMS': 1000,

            'form-0-datetime_0': '1',
            'form-0-datetime_1': '1',
            'form-0-datetime_2': '2015',
            'form-0-datetime_3': '9',
            'form-0-datetime_4': '0',
            'form-0-datetime_5': 'UTC',

            'form-1-datetime_0': '3',
            'form-1-datetime_1': '3',
            'form-1-datetime_2': '2016',
            'form-1-datetime_3': '9',
            'form-1-datetime_4': '0',
            'form-1-datetime_5': 'UTC',
        }

        with self.login(self.regular):
            self.get('generic:permissions:edit')
            self.response_403()
            self.post('generic:permissions:edit', data=data)
            self.response_403()

        with self.login(self.staff):
            self.assertGoodView('generic:permissions:edit')
            self.post('generic:permissions:edit', data=data)
            self.response_302()

        self.assertEqual(2, TestDateTimeField.objects.count())



from django.template import Context, Template
from example_app.tests.factories import RelativeFactory
from test_plus import TestCase


class RelatedFilterTest(TestCase):

    def setUp(self):
        RelativeFactory.reset_sequence()
        self.template = Template('''
        {% load common mvp_tags %}
        {% spaceless %}
            <dl>
                {% for manager, name in object|related:related %}
                    <dt>{{ name }}</dt>
                    <dd>{{ manager|type }} ({{ manager.count }})</dd>
                {% endfor %}
            </dl>
        {% endspaceless %}
        ''')

    def _get_context(self, **kwargs):
        return Context(kwargs)

    def test_related_no_whitelist(self):
        rel = RelativeFactory()
        context = self._get_context(object=rel.link)
        output = self.template.render(context).strip()
        self.assertEqual(output, '<dl></dl>')

    def test_related_empty_whitelist(self):
        rel = RelativeFactory()
        context = self._get_context(object=rel.link, related=())
        output = self.template.render(context).strip()
        self.assertEqual(output, '<dl></dl>')

    def test_related_specific_whitelist(self):
        rel = RelativeFactory()
        context = self._get_context(
            object=rel.link,
            related=('relative_set',),
        )
        output = self.template.render(context).strip()
        self.assertEqual(output, '<dl>'
                                 '<dt>relative_set</dt>'
                                 '<dd>RelatedManager (1)</dd>'
                                 '</dl>')



import os.path
import tempfile

from django.test import TestCase
from touchtechnology.common.storage import FileSystemStorage


class StorageTests(TestCase):

    def setUp(self):
        location = tempfile.mkdtemp()
        self.storage = FileSystemStorage(location=location)

    def test_makedirs(self):
        path, folder = '', 'test_mkdir_folder/with_two/subfolders'
        self.assertEqual(self.storage.listdir(path), ([], []))
        self.storage.makedirs(folder)
        self.assertEqual(
            self.storage.listdir(os.path.dirname(folder)),
            ([os.path.basename(folder)], []))
        self.assertEqual(
            self.storage.listdir(
                os.path.dirname(os.path.dirname(folder))),
            ([os.path.basename(os.path.dirname(folder))], []))

    def test_mkdir(self):
        path, folder = '', 'test_mkdir_folder'
        self.assertEqual(self.storage.listdir(path), ([], []))
        self.storage.mkdir(folder)
        self.assertEqual(self.storage.listdir(path), ([folder], []))
        self.assertEqual(self.storage.listdir(folder), ([], []))

    def test_mkdir_path(self):
        path, folder = 'path', 'test_mkdir_path_folder'
        self.storage.mkdir(path)
        self.assertEqual(self.storage.listdir(path), ([], []))
        self.storage.mkdir(folder, path)
        self.assertEqual(self.storage.listdir(path), ([folder], []))
        self.assertEqual(
            self.storage.listdir(os.path.join(path, folder)),
            ([], []))

    def test_path(self):
        filename = 'test_path/to/file.txt'
        self.assertEqual(
            self.storage.path(filename),
            os.path.join(self.storage.location, filename))

    def test_rmdir(self):
        path, folder = '', 'test_rmdir'
        with self.assertRaises(OSError) as cm:
            self.storage.rmdir(folder)
        self.assertEqual(cm.exception.errno, 2)
        self.storage.mkdir(folder)
        self.assertEqual(self.storage.listdir(path), ([folder], []))
        self.storage.rmdir(folder)
        self.assertEqual(self.storage.listdir(path), ([], []))

    def test_rmdir_subdir(self):
        folder = 'test_rmdir_subdir'
        subdir = os.path.join(folder, 'subdir')
        self.storage.makedirs(subdir)
        with self.assertRaises(OSError):
            self.storage.rmdir(folder)

    def test_writable(self):
        folder = 'test_writable'
        with self.assertRaises(OSError) as cm:
            self.storage.writable(folder)
        self.assertEqual(cm.exception.errno, 2)
        self.storage.mkdir(folder)
        self.assertTrue(self.storage.writable(folder))



from __future__ import unicode_literals

import pytz
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.http import is_safe_url, urlencode
from django.utils.six.moves.urllib.parse import parse_qsl, urlsplit, urlunsplit
from django.views.decorators.http import require_POST
from touchtechnology.common.forms.tz import SelectTimezoneForm


def login(request, to, *args, **kwargs):
    scheme, netloc, path, query, fragments = urlsplit(to)
    query = dict(parse_qsl(query))
    query.update(kwargs)
    query = urlencode(query)
    to = urlunsplit((scheme, netloc, path, query, fragments))
    return redirect(to)


@require_POST
def set_timezone(request):
    url = request.META.get('HTTP_REFERER', '/')
    response = HttpResponseRedirect(url)

    form = SelectTimezoneForm(data=request.POST)
    if form.is_valid():
        tzname = form.cleaned_data.get('timezone')
        if tzname in pytz.all_timezones_set:
            if hasattr(request, 'session'):
                request.session['django_timezone'] = tzname
            else:
                response.set_cookie('django_timezone', tzname)

    if not is_safe_url(url=url, host=request.get_host()):
        raise Http404

    return response



import os.path

from django import forms
from django.contrib.contenttypes.models import ContentType
from django.core.files.storage import default_storage
from django.forms.models import inlineformset_factory
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _
from modelforms.forms import ModelForm
from touchtechnology.common.default_settings import (
    SITEMAP_EDIT_PARENT, SITEMAP_HTTPS_OPTION, SITEMAP_ROOT,
)
from touchtechnology.common.forms.mixins import BootstrapFormControlMixin, SuperUserSlugMixin
from touchtechnology.common.models import SitemapNode
from touchtechnology.content.app_settings import PAGE_CONTENT_BLOCKS
from touchtechnology.content.models import (
    Content, NodeContent, Page, PageContent, Placeholder, Redirect,
)


class PlaceholderChoiceField(forms.ModelChoiceField):
    def clean(self, value):
        return super(PlaceholderChoiceField, self).clean(value).pk

    def label_from_instance(self, obj):
        try:
            cls = import_string(obj.path)
        except (ImportError, ValueError):
            return obj.path
        return cls.verbose_name()


class ParentChildModelForm(BootstrapFormControlMixin, ModelForm):
    """
    ``ModelForm`` subclass that can be used to tie together the editing of a
    parent/child relationship in a single form.

    Typical use will be ``SitemapNode`` and any model which is connected by a
    generic foriegn key relationship.
    """
    def __init__(self, parent_form_class, parent_form_kwargs=None,
                 child_attr='object', instance=None, *args, **kwargs):
        # we are initiated with the parent instance, but this form is
        # fundamentally that of the child form; track the parent.
        parent = instance if instance is not None else None
        instance = getattr(parent, child_attr, None)
        self.child_attr = child_attr

        # initialise with our newly obtained child instance
        super(ParentChildModelForm, self).__init__(
            instance=instance, *args, **kwargs)

        # determine if there are any keyword arguments for the parent form to
        # be initialised with and establish the form instance
        pfkw = parent_form_kwargs if parent_form_kwargs else {}
        self.parent_form = parent_form_class(
            instance=parent,
            data=self.data if self.is_bound else None,
            prefix='parent-%s' % self.prefix,
            **pfkw
        )

    def is_valid(self):
        """
        Ensure both the regular ModelForm and the parent_form are valid.
        """
        result = super(ParentChildModelForm, self).is_valid()
        if self.is_bound:
            result = result and self.parent_form.is_valid()
        return result

    def save(self, commit=True):
        """
        Respond with the ``parent_form`` saved instance.
        """
        child = super(ParentChildModelForm, self).save(commit=commit)
        setattr(self.parent_form.instance, self.child_attr, child)
        return self.parent_form.save(commit=commit)

    def __iter__(self):
        """
        Iterate through the fields from the ``parent_form`` first, then
        proceed onto the fields explicitly available on this form.
        """
        for name in self.parent_form.fields:
            yield self.parent_form[name]
        for name in self.fields:
            yield self[name]

    def _get_media(self):
        return super(ParentChildModelForm, self).media + \
            self.parent_form.media
    media = property(_get_media)


class BaseSitemapNodeForm(SuperUserSlugMixin, ModelForm):
    def __init__(self, *args, **kwargs):
        super(BaseSitemapNodeForm, self).__init__(*args, **kwargs)
        if not self.instance.level and self.instance.slug == SITEMAP_ROOT:
            self.fields.pop('slug', None)
            self.fields.pop('slug_locked', None)
        if not self.fields['restrict_to_groups'].queryset.count():
            self.fields.pop('restrict_to_groups', None)
        if not SITEMAP_HTTPS_OPTION:
            self.fields.pop('require_https', None)

    class Meta:
        model = SitemapNode
        fields = (
            'parent',
            'title',
            'short_title',
            'enabled',
            'hidden_from_navigation',
            'hidden_from_sitemap',
            'require_https',
            'restrict_to_groups',
            'slug',
            'slug_locked',
        )


class NewSitemapNodeForm(BaseSitemapNodeForm):
    pass


class SitemapNodeForm(BaseSitemapNodeForm):
    def __init__(self, *args, **kwargs):
        super(SitemapNodeForm, self).__init__(*args, **kwargs)
        if not SITEMAP_EDIT_PARENT and self.instance.pk:
            self.fields.pop('parent', None)
        elif self.instance.pk:
            # update the queryset to exclude the node itself, as well as any
            # nodes which are descendants of itself -- these will both result
            # in an invalid move in the tree.
            nodes = self.fields['parent'].queryset
            nodes = nodes.exclude(pk=self.instance.pk)
            nodes = nodes.exclude(
                tree_id=self.instance.tree_id, level__gt=self.instance.level)
            self.fields['parent'].queryset = nodes


class PlaceholderSitemapNodeForm(SitemapNodeForm):
    object_id = PlaceholderChoiceField(
        queryset=Placeholder.objects.all(),
        label=_("Application"),
    )

    def __init__(self, *args, **kwargs):
        super(PlaceholderSitemapNodeForm, self).__init__(*args, **kwargs)
        if not self.instance.content_type:
            self.instance.content_type = \
                ContentType.objects.get_for_model(Placeholder)

    class Meta(SitemapNodeForm.Meta):
        fields = ('object_id',) + SitemapNodeForm.Meta.fields


class NewPlaceholderSitemapNodeForm(PlaceholderSitemapNodeForm):
    class Meta(PlaceholderSitemapNodeForm.Meta):
        fields = ('parent',) + PlaceholderSitemapNodeForm.Meta.fields
        exclude = None


class PlaceholderConfigurationBase(forms.Form):

    def __init__(self, user, instance, *args, **kwargs):
        self.user = user
        self.instance = instance
        initial = kwargs.pop('initial', {})
        initial.update(instance.kwargs)
        super(PlaceholderConfigurationBase, self).__init__(
            initial=initial, *args, **kwargs)

    def save(self, *args, **kwargs):
        self.instance.kwargs = self.cleaned_data
        self.instance.save()
        return self.instance.kwargs


PlaceholderContentFormset = inlineformset_factory(
    SitemapNode, NodeContent, fields=('copy',),
    max_num=1, can_delete=False)


class PageForm(ParentChildModelForm):
    class Meta:
        model = Page
        fields = ('template', 'keywords', 'description')


class PageContentForm(BootstrapFormControlMixin, ModelForm):
    class Meta:
        model = PageContent
        fields = ('copy', 'label', 'sequence')
        widgets = {
            'sequence': forms.HiddenInput(),
        }


BasePageContentFormset = inlineformset_factory(
    Page, PageContent, form=PageContentForm, can_delete=False, extra=0)


class PageContentFormset(BasePageContentFormset):
    def __init__(self, instance=None, *args, **kwargs):
        self.node = None
        if instance is not None:
            self.node = instance
            instance = instance.object
        super(PageContentFormset, self).__init__(
            instance=instance, *args, **kwargs)

    def total_form_count(self):
        if isinstance(PAGE_CONTENT_BLOCKS, str):
            # We may want to overload this by tenant.
            callback = import_string(PAGE_CONTENT_BLOCKS)
            return callback(self.node)
        return PAGE_CONTENT_BLOCKS

    def _construct_form(self, i, **kwargs):
        return super(PageContentFormset, self)._construct_form(
            i, empty_permitted=False, initial={'sequence': i + 1})


class ContentForm(BootstrapFormControlMixin, ModelForm):
    class Meta:
        model = Content
        fields = ('copy',)


class RedirectEditForm(BootstrapFormControlMixin, ModelForm):
    class Meta:
        model = Redirect
        fields = (
            'source_url',
            'destination_url',
            'label',
            'active',
            'permanent',
        )


class FileFormMixin(BootstrapFormControlMixin):
    def __init__(self, path, *args, **kwargs):
        super(FileFormMixin, self).__init__(*args, **kwargs)
        self.path = path


class NewFolderForm(FileFormMixin, forms.Form):

    name = forms.SlugField(max_length=30, required=False,
                           label=_("New folder name"))

    def clean_name(self):
        name = self.cleaned_data.get('name')
        if self.path is None:
            path = name
        else:
            path = os.path.join(self.path, name)
        if name and default_storage.exists(path):
            raise forms.ValidationError(_('Already exists.'))
        return name

    def save(self, *args, **kwargs):
        return default_storage.mkdir(
            name=self.cleaned_data['name'], path=self.path)


class FileUploadForm(FileFormMixin, forms.Form):

    file = forms.FileField(label=_("File to upload"), required=False)

    def save(self, *args, **kwargs):
        f = self.cleaned_data.get('file')
        name = os.path.join(self.path or '', f.name)
        default_storage.save(name, f)
        return name



from __future__ import unicode_literals

import magic
from dateutil.relativedelta import relativedelta
from django.conf.urls import include, url
from django.contrib.syndication.views import Feed
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.feedgenerator import Atom1Feed, Rss201rev2Feed
from touchtechnology.common.sites import Application
from touchtechnology.news.app_settings import PAGINATE_BY
from touchtechnology.news.decorators import (
    date_view, last_modified_article, news_last_modified,
)
from touchtechnology.news.forms import ConfigurationForm
from touchtechnology.news.models import Article, Category

YEAR_DELTA = relativedelta(years=1)
MONTH_DELTA = relativedelta(months=1)
DAY_DELTA = relativedelta(days=1)


class NewsSite(Application):

    kwargs_form_class = ConfigurationForm

    def __init__(self, name='news', app_name='news', **kwargs):
        self.node = kwargs.get('node')  # store the node for future reference
        super(NewsSite, self).__init__(name=name, app_name=app_name, **kwargs)

    def get_urls(self):
        urls = [
            url(r'^archive/$', self.archive_index, name='archive'),
            url(r'^(?P<year>\d{4})/', include([
                url(r'^$', self.archive_year, name='year'),
                url(r'^(?P<month>[a-z]{3})/', include([
                    url(r'^$', self.archive_month, name='month'),
                    url(r'^(?P<day>\d{1,2})/', include([
                        url(r'^$', self.archive_day, name='day'),
                        url(r'^(?P<slug>[^/]+)/$',
                            self.article, name='article'),
                    ])),
                ])),
            ])),
        ]

        feeds = [
            url(r'^feeds/', include([
                url(r'^$', self.feeds, name='feeds'),
                url(r'^(?P<format>(atom|rss))/$',
                    self.feed, name='feed'),
            ])),
        ]

        if self.kwargs:
            urlpatterns = [
                url(r'^$', self.list_articles, kwargs=self.kwargs,
                    name='category-index'),
                url(r'^', include(urls), kwargs=self.kwargs),
                url(r'^', include(feeds)),
            ]
        else:
            urlpatterns = [
                url(r'^$', self.index, name='index'),
                url(r'^', include(urls)),
                url(r'^', include(feeds)),
                url(r'^(?P<category>[-\w]+)/$', self.list_articles,
                    name='category-index'),
            ]

        return urlpatterns

    @property
    def urls(self):
        return self.get_urls(), self.app_name, self.name

    @property
    def categories(self):
        return Category.objects.all()

    @news_last_modified
    def index(self, request, category=None, *args, **kwargs):
        context = {
            'object_list': Article.objects.live()[:PAGINATE_BY],
        }
        context.update(kwargs)
        return self.render(request, 'touchtechnology/news/index.html', context)

    @news_last_modified
    def list_articles(self, request, category=None, *args, **kwargs):
        articles = Article.objects.live()
        templates = self.template_path('list_articles.html', category)

        extra_context = {}
        extra_context.update(kwargs)

        if category is not None:
            category = get_object_or_404(Category, slug=category)
            articles = articles.filter(categories=category)
            extra_context['category'] = category

        context = {
            'object_list': articles,
        }
        context.update(extra_context)

        return self.render(request, templates, context)

    @date_view
    @last_modified_article
    def article(self, request, date, slug, category=None, *args, **kwargs):
        date_range = (date, date + DAY_DELTA)
        article = get_object_or_404(Article, slug=slug,
                                    published__range=date_range)
        context = {
            'article': article,
            'category': category,
        }
        context.update(kwargs)
        templates = self.template_path('article.html', category)
        return self.render(request, templates, context)

    @date_view
    @last_modified_article
    def archive_day(self, request, date, category=None, *args, **kwargs):
        queryset = Article.objects.live().filter(
            published__range=(date, date + DAY_DELTA))
        context = {
            'day': date,
            'object_list': queryset,
        }
        context.update(kwargs)
        templates = ['touchtechnology/news/archive/day.html']
        return self.render(request, templates, context)

    @date_view
    @last_modified_article
    def archive_month(self, request, date, category=None, *args, **kwargs):
        queryset = Article.objects.live().filter(
            published__range=(date, date + MONTH_DELTA))
        try:
            date_list = queryset.datetimes('published', 'day',
                                           tzinfo=timezone.utc)
        except AttributeError:
            date_list = queryset.dates('published', 'day')
        context = {
            'date_list': date_list,
            'month': date,
            'object_list': queryset,
        }
        context.update(kwargs)
        templates = ['touchtechnology/news/archive/month.html']
        return self.render(request, templates, context)

    @date_view
    @last_modified_article
    def archive_year(self, request, date, category=None, *args, **kwargs):
        queryset = Article.objects.live().filter(
            published__range=(date, date + YEAR_DELTA))

        try:
            date_list = queryset.datetimes('published', 'month',
                                           tzinfo=timezone.utc)
        except AttributeError:
            date_list = queryset.dates('published', 'month')

        # compatibility with date_based.archive_year
        year = str(date.year)

        context = {
            'date_list': date_list,
            'year': year,
            'object_list': queryset,
        }
        context.update(kwargs)

        templates = ['touchtechnology/news/archive/year.html']
        return self.render(request, templates, context)

    def archive_index(self, request, *args, **kwargs):
        queryset = Article.objects.live()
        try:
            date_list = queryset.datetimes('published', 'year',
                                           tzinfo=timezone.utc)
        except AttributeError:
            date_list = queryset.dates('published', 'year')
        context = {
            'date_list': date_list,
            'latest': queryset.order_by('-published')[:15],
        }
        context.update(kwargs)
        templates = ['touchtechnology/news/archive/index.html']
        return self.render(request, templates, context)

    def feeds(self, request, *args, **kwargs):
        context = {
            'feeds': {
                'ATOM': 'atom',
                'RSS': 'rss',
            },
        }
        context.update(kwargs)
        templates = self.template_path('feeds.html')
        return self.render(request, templates, context)

    def feed(self, request, format, category=None, *args, **kwargs):
        class NewsFeed(Feed):
            title = kwargs.get('node') or getattr(request, 'tenant', None)
            link = self.reverse('feeds')

            def items(slf):
                live = Article.objects.live()
                if category is not None:
                    live = live.filter(categories__slug=category)
                return live.prefetch_related('categories')

            def item_categories(slf, item):
                return item.categories.values_list('title', flat=True)

            def item_link(slf, item):
                date = item.published.date()
                args = (date.year, date.strftime('%b').lower(),
                        date.day, item.slug)
                return self.reverse('article', args=args)

            def item_pubdate(slf, item):
                return item.published

            def item_enclosure_url(slf, item):
                try:
                    return item.image.url
                except Exception:
                    pass

            def item_enclosure_length(slf, item):
                try:
                    return item.image.size
                except Exception:
                    pass

            def item_enclosure_mime_type(slf, item):
                try:
                    return magic.from_file(item.image.path, mime=True)
                except Exception:
                    pass


        class AtomNewsFeed(NewsFeed):
            feed_type = Atom1Feed

        class RssNewsFeed(NewsFeed):
            feed_type = Rss201rev2Feed

        feed_class = {
            'atom': AtomNewsFeed,
            'rss': RssNewsFeed,
        }
        return feed_class[format]()(request)


news = NewsSite()



import touchtechnology.admin
from django.template import Library

register = Library()


@register.inclusion_tag('touchtechnology/admin/_version.html',
                        takes_context=True)
def version_string(context):
    component = context.get('component')
    if component is not None:
        return {'version': component.version}
    node = context.get('node')
    if node is not None and hasattr(node.object, 'site'):
        return {'version': node.object.site(node).version}
    return {'version': touchtechnology.admin.__version__}


@register.filter
def allowed(components, request):
    """
    Given a list of components, return only those which are accessible to the
    specified user. Used to hide options from the tab-bar in the admin UI.
    """
    def show(t):
        __, module, __, instance, schemas = t

        # When django-tenant-schemas is in operation, show a component based
        # on whether it has been associated with the active schema.
        if hasattr(request, 'tenant'):
            if schemas is not None:
                if request.tenant.schema_name not in schemas:
                    # Warning: this is not going to protect the component from
                    # unauthorised use; it must also ensure that permissions
                    # are enforced. Superuser is *very* special and will always
                    # have permission, so do not grant this level willy-nilly.
                    return False

        if request.user.has_module_perms(module):
            return True

        if request.user.userobjectpermission_set.filter(
                permission__content_type__app_label=module):
            return True

        if request.user.groups.filter(
                permissions__content_type__app_label=module).distinct():
            return True

        # FIXME once all applications are protected, swap this for False.
        return instance.unprotected

    return [c for c in components if show(c)]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-27 21:16
from __future__ import unicode_literals

import django.contrib.postgres.fields
import django.db.models.deletion
import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0025_matchvideo_baseline'),
    ]

    operations = [
        migrations.AddField(
            model_name='match',
            name='videos',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.URLField(), null=True, size=None),
        ),
        migrations.AlterField(
            model_name='matchvideo',
            name='match',
            field=touchtechnology.common.db.models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='competition.Match'),
        ),
    ]



from django.test.utils import override_settings
from test_plus import TestCase


@override_settings(ROOT_URLCONF='example_app.urls')
class DateTimeFieldTest(TestCase):

    def test_validate_form_output_tz(self):
        text = """
        <optgroup label="Australia">
            <option value="Australia/Adelaide">Adelaide</option>
            <option value="Australia/Brisbane">Brisbane</option>
            <option value="Australia/Broken_Hill">Broken Hill</option>
            <option value="Australia/Currie">Currie</option>
            <option value="Australia/Darwin">Darwin</option>
            <option value="Australia/Eucla">Eucla</option>
            <option value="Australia/Hobart">Hobart</option>
            <option value="Australia/Lindeman">Lindeman</option>
            <option value="Australia/Lord_Howe">Lord Howe</option>
            <option value="Antarctica/Macquarie">Macquarie</option>
            <option value="Australia/Melbourne">Melbourne</option>
            <option value="Australia/Perth">Perth</option>
            <option value="Australia/Sydney">Sydney</option>
        </optgroup>
        """
        with self.settings(USE_TZ=False):
            self.get('datetime:index')
            self.assertResponseNotContains(text)

        with self.settings(USE_TZ=True):
            self.get('datetime:index')
            self.assertResponseContains(text)

    def test_submit_date_time_valid(self):
        data = {
            'form-datetime_0': '1',
            'form-datetime_1': '1',
            'form-datetime_2': '2014',
            'form-datetime_3': '0',
            'form-datetime_4': '0',

            'formset-TOTAL_FORMS': '0',
            'formset-INITIAL_FORMS': '0',
            'formset-MAX_NUM_FORMS': '1000',
        }

        with self.settings(USE_TZ=False):
            self.post('datetime:index', data=data)
            self.response_302()

        data.update({
            'form-datetime_5': 'Australia/Sydney',
        })

        with self.settings(USE_TZ=True):
            self.post('datetime:index', data=data)
            self.response_302()

    def test_submit_date_time_blank(self):
        data = {
            'form-datetime_0': '',
            'form-datetime_1': '',
            'form-datetime_2': '',
            'form-datetime_3': '',
            'form-datetime_4': '',

            'formset-TOTAL_FORMS': '0',
            'formset-INITIAL_FORMS': '0',
            'formset-MAX_NUM_FORMS': '1000',
        }

        with self.settings(USE_TZ=False):
            self.post('datetime:index', data=data)
            self.response_302()

        data.update({
            'form-datetime_5': '',
        })

        with self.settings(USE_TZ=True):
            self.post('datetime:index', data=data)
            self.response_302()



from django.conf import settings
from django.utils.translation import ugettext_lazy as _

__all__ = (
    'CONTENT_LABEL_CHOICES',
    'DETAIL_IMAGE_KWARGS',
    'DETAIL_IMAGE_PROCESSORS',
    'NEWS_CONTENT_BLOCKS',
    'PAGINATE_BY',
    'THUMBNAIL_IMAGE_KWARGS',
    'THUMBNAIL_IMAGE_PROCESSORS',
)

S = lambda n, d=None: getattr(settings, 'TOUCHTECHNOLOGY_NEWS_' + n, d) or d

DEFAULT_CONTENT_LABEL_CHOICES = (
    ('copy', _('Copy')),
)

DEFAULT_DETAIL_IMAGE_KWARGS = {}
DEFAULT_DETAIL_IMAGE_PROCESSORS = (
    ('pilkit.processors.resize.SmartResize', (320, 240), {}),
)

DEFAULT_THUMBNAIL_IMAGE_KWARGS = {}
DEFAULT_THUMBNAIL_IMAGE_PROCESSORS = (
    ('pilkit.processors.resize.SmartResize', (160, 120), {}),
)

CONTENT_LABEL_CHOICES = S('CONTENT_LABEL_CHOICES',
                          DEFAULT_CONTENT_LABEL_CHOICES)
DETAIL_IMAGE_KWARGS = S('DETAIL_IMAGE_KWARGS',
                        DEFAULT_DETAIL_IMAGE_KWARGS)
DETAIL_IMAGE_PROCESSORS = S('DETAIL_IMAGE_PROCESSORS',
                            DEFAULT_DETAIL_IMAGE_PROCESSORS)
NEWS_CONTENT_BLOCKS = S('CONTENT_BLOCKS', 1)
PAGINATE_BY = S('PAGINATE_BY', 5)
THUMBNAIL_IMAGE_KWARGS = S('THUMBNAIL_IMAGE_KWARGS',
                           DEFAULT_THUMBNAIL_IMAGE_KWARGS)
THUMBNAIL_IMAGE_PROCESSORS = S('THUMBNAIL_IMAGE_PROCESSORS',
                               DEFAULT_THUMBNAIL_IMAGE_PROCESSORS)



import six
from django.test.utils import override_settings
from test_plus import TestCase
from tournamentcontrol.competition.tests import factories

ORIGINAL_BEHAVIOUR = 'tournamentcontrol.competition.rank.points_func'
CORRECT_BEHAVIOUR = 'tournamentcontrol.competition.rank.correct_points_func'


class RankingLadderEntryTests(TestCase):
    """
    The default implementation awards the following bonus multipliers:

    -   double the win points for wins > 15
    -   win points for wins > 10
    -   half win points for wins > 5
    -   half loss points for losses < 2

    The DYNAMIC_FIELDS are computed with annotations in the LadderEntryQuerySet
    using the function addressed by the `TOURNAMENTCONTROL_RANK_POINTS_FUNC`
    setting.

    Our original implementation had a bug where margin bonuses for winning
    would be accumulated if more than one condition was satisfied. We test both
    the original and desired behaviour.
    """
    DYNAMIC_FIELDS = ('margin', 'diff', 'rank_points')

    def assertLadderEntries(self, home_score, away_score, expected):
        match = factories.MatchFactory.create(
            rank_importance=1,
            home_team_score=home_score,
            away_team_score=away_score,
        )
        six.assertCountEqual(
            self,
            match.ladder_entries.values_list(*self.DYNAMIC_FIELDS),
            expected,
        )

    # Original Behaviour

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=ORIGINAL_BEHAVIOUR)
    def test_bug_win_no_margin_bonus(self):
        "win by <=5, loss by >=2"
        self.assertLadderEntries(3, 1, [(2, 2, 15), (2, -2, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=ORIGINAL_BEHAVIOUR)
    def test_bug_win_margin_bonus_small(self):
        "win by >5"
        self.assertLadderEntries(7, 1, [(6, 6, 22.5), (6, -6, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=ORIGINAL_BEHAVIOUR)
    def test_bug_win_margin_bonus_medium(self):
        "win by >10"
        self.assertLadderEntries(12, 1, [(11, 11, 37.5), (11, -11, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=ORIGINAL_BEHAVIOUR)
    def test_bug_win_margin_bonus_large(self):
        "win by >15"
        self.assertLadderEntries(17, 1, [(16, 16, 67.5), (16, -16, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=ORIGINAL_BEHAVIOUR)
    def test_bug_loss_margin_bonus(self):
        "loss by <2"
        self.assertLadderEntries(2, 1, [(1, 1, 15), (1, -1, 7.5)])

    # Correct Behaviour

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=CORRECT_BEHAVIOUR)
    def test_win_no_margin_bonus(self):
        "win by <=5, loss by >=2"
        self.assertLadderEntries(3, 1, [(2, 2, 15), (2, -2, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=CORRECT_BEHAVIOUR)
    def test_win_margin_bonus_small(self):
        "win by >5"
        self.assertLadderEntries(7, 1, [(6, 6, 22.5), (6, -6, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=CORRECT_BEHAVIOUR)
    def test_win_margin_bonus_medium(self):
        "win by >10"
        self.assertLadderEntries(12, 1, [(11, 11, 30), (11, -11, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=CORRECT_BEHAVIOUR)
    def test_win_margin_bonus_large(self):
        "win by >15"
        self.assertLadderEntries(17, 1, [(16, 16, 45), (16, -16, 5)])

    @override_settings(TOURNAMENTCONTROL_RANK_POINTS_FUNC=CORRECT_BEHAVIOUR)
    def test_loss_margin_bonus(self):
        "loss by <2"
        self.assertLadderEntries(2, 1, [(1, 1, 15), (1, -1, 7.5)])



import factory
from touchtechnology.content.models import Redirect


class RedirectFactory(factory.django.DjangoModelFactory):

    class Meta:
        model = Redirect

    source_url = factory.Faker('uri_path')
    destination_url = factory.Faker('uri_path')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('headline', models.CharField(max_length=150, verbose_name='Headline')),
                ('abstract', models.TextField(verbose_name='Abstract')),
                ('published', touchtechnology.common.db.models.DateTimeField(help_text='Set a date & time in the future to schedule an announcement.', verbose_name='Published')),
                ('slug', models.SlugField(max_length=150, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Locked Slug')),
                ('byline', models.CharField(max_length=75, verbose_name='Byline', blank=True)),
                ('keywords', models.CharField(max_length=255, verbose_name='Keywords', blank=True)),
                ('is_active', models.BooleanField(default=True, verbose_name='Enabled')),
                ('image', models.ImageField(upload_to=b'news', null=True, verbose_name='Image', blank=True)),
            ],
            options={
                'ordering': ('-published',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ArticleContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.SlugField(max_length=20, verbose_name='CSS class', choices=[(b'copy', 'Copy')])),
                ('sequence', models.PositiveIntegerField(verbose_name='Sequence')),
                ('copy', touchtechnology.common.db.models.HTMLField(verbose_name='Copy', blank=True)),
                ('article', models.ForeignKey(related_name=b'content', verbose_name='Article', to='news.Article', on_delete=models.PROTECT)),
            ],
            options={
                'ordering': ('sequence',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=75, verbose_name='Title')),
                ('short_title', models.CharField(help_text='Used in navigation, a shorter alternative to the main title.', max_length=50, verbose_name='Short Title', blank=True)),
                ('slug', models.SlugField(max_length=75, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Locked Slug')),
                ('is_active', models.BooleanField(default=True, verbose_name=b'Enabled')),
                ('hidden_from_navigation', models.BooleanField(default=False, help_text="When set to 'Yes' this object will still be available, but will not appear in menus.", verbose_name='Hide from menus')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='article',
            name='categories',
            field=models.ManyToManyField(to='news.Category', verbose_name='Categories', blank=True),
            preserve_default=True,
        ),
    ]



import datetime
import logging
import time

from dateutil.parser import parse
from django.db.models import Count
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.utils.functional import wraps
from tournamentcontrol.competition.models import Club, Competition, Season

try:
    from urllib.parse import ParseResult
except ImportError:
    from urlparse import ParseResult


logger = logging.getLogger(__name__)


@method_decorator
def registration(f, *a, **kw):
    @wraps(f)
    def _decorated(*args, **kwargs):
        club_id = kwargs.pop('club_id', None)
        person_id = kwargs.pop('person_id', None)
        season_id = kwargs.pop('season_id', None)
        team_id = kwargs.pop('team_id', None)

        if club_id:
            club = get_object_or_404(Club, pk=club_id)
            kwargs['club'] = club

            if person_id:
                person = get_object_or_404(club.members, pk=person_id)
                kwargs['person'] = person

            if season_id:
                season = get_object_or_404(Season, pk=season_id,
                                           competition__clubs=club)
                kwargs['season'] = season

                # Knowing the season implies the competition
                kwargs['competition'] = season.competition

                # List of teams for the season.
                kwargs['teams'] = club.teams.filter(division__season=season)

            if team_id:
                team = get_object_or_404(club.teams, pk=team_id)
                kwargs['team'] = team

                # Knowing the team implies the division, season, competition.
                kwargs['division'] = team.division
                kwargs['season'] = team.division.season
                kwargs['competition'] = team.division.season.competition

        extra_context = kwargs.pop('extra_context', {})
        extra_context.update(kwargs)
        kwargs['extra_context'] = extra_context

        return f(*args, **kwargs)
    return _decorated


def competition_by_pk(f, *a, **kw):
    @wraps(f)
    def _decorated(request, *args, **kwargs):
        competition_id = kwargs.pop('competition_id', None)
        division_id = kwargs.pop('division_id', None)
        season_id = kwargs.pop('season_id', None)
        stage_id = kwargs.pop('stage_id', None)
        team_id = kwargs.pop('team_id', None)
        venue_id = kwargs.pop('venue_id', None)
        ground_id = kwargs.pop('ground_id', None)
        pool_id = kwargs.pop('pool_id', None)
        match_id = kwargs.pop('match_id', None)

        datestr = kwargs.pop('datestr', None)
        timestr = kwargs.pop('timestr', None)

        if competition_id:
            competition = get_object_or_404(Competition, pk=competition_id)
            kwargs['competition'] = competition
            if season_id:
                season = get_object_or_404(competition.seasons.select_related('competition'), pk=season_id)
                kwargs['season'] = season
                if division_id:
                    division = get_object_or_404(season.divisions.select_related('season__competition'),
                                                 pk=division_id)
                    kwargs['division'] = division
                    if stage_id:
                        stage = get_object_or_404(division.stages.select_related('division__season__competition'),
                                                  pk=stage_id)
                        kwargs['stage'] = stage
                        if team_id:
                            team = get_object_or_404(stage.undecided_teams.select_related('stage__division__season__competition'),
                                                     pk=team_id)
                            kwargs['team'] = team

                        if pool_id:
                            pool = get_object_or_404(stage.pools.select_related('stage__division__season__competition'),
                                                     pk=pool_id)
                            kwargs['pool'] = pool

                        if match_id:
                            match = get_object_or_404(stage.matches.select_related('stage__division__season__competition'),
                                                      pk=match_id)
                            kwargs['match'] = match

                    elif team_id:
                        team = get_object_or_404(division.teams.select_related('division__season__competition'),
                                                 pk=team_id)
                        kwargs['team'] = team
                elif venue_id:
                    venue = get_object_or_404(season.venues.select_related('season__competition'),
                                              pk=venue_id)
                    kwargs['venue'] = venue
                    if ground_id:
                        ground = get_object_or_404(venue.grounds.select_related('venue__season__competition'),
                                                   pk=ground_id)
                        kwargs['ground'] = ground

                        if match_id:
                            match = get_object_or_404(ground.matches,
                                                      pk=match_id)
                            kwargs['match'] = match
                    elif match_id:
                        match = get_object_or_404(venue.matches,
                                                  pk=match_id)
                        kwargs['match'] = match

        if datestr:
            kwargs['date'] = parse(datestr).date()
            if timestr:
                kwargs['time'] = datetime.time(
                    *time.strptime(timestr, '%H%M')[3:5])

        kwargs['base_url'] = ParseResult(
            'https' if request.is_secure() else 'http',
            request.get_host(), '/', '', '', '').geturl()

        extra_context = kwargs.pop('extra_context', {})
        extra_context.update(kwargs)
        kwargs['extra_context'] = extra_context

        return f(request, *args, **kwargs)
    return _decorated
competition = method_decorator(competition_by_pk)


def competition_by_slug(f, *a, **kw):
    @wraps(f)
    def _decorated(*args, **kwargs):
        # deep dive into the object internals and fetch the Application
        # instance so we can call the sitemapnodes method.
        try:
            manager = f.__closure__[1].cell_contents._competitions
            # manager = f.func_closure[1].cell_contents._competitions
        # except AttributeError:
        #     manager = f.__closure__[1].cell_contents._competitions
        except TypeError:
            manager = Competition.objects

        club_slug = kwargs.pop('club', None)
        competition_slug = kwargs.pop('competition', None)
        division_slug = kwargs.pop('division', None)
        pool_slug = kwargs.pop('pool', None)
        season_slug = kwargs.pop('season', None)
        stage_slug = kwargs.pop('stage', None)
        team_slug = kwargs.pop('team', None)
        match_pk = kwargs.pop('match', None)

        base = manager.prefetch_related(
            'seasons',
            'seasons__divisions',
            'seasons__divisions__stages',
            'seasons__divisions__teams',
        )

        # see if there is an original node, we might need it later
        o_node = kwargs.pop('node', None)

        if competition_slug:
            competition = get_object_or_404(base, slug=competition_slug)
            kwargs['competition'] = competition
            if season_slug:
                season = get_object_or_404(competition.seasons,
                                           slug=season_slug)
                kwargs['season'] = season

                # List of clubs participating in this season.
                kwargs['clubs'] = competition.clubs.filter(
                    teams__division__season=season).distinct()

                # We should deprecate this if it is unused in templates.
                kwargs['other_seasons'] = competition.seasons.exclude(
                    slug=season_slug)

                # So we can build a navigation hierarchy, select extra column
                # which tells us if the season in a list of seasons is the
                # current scope.
                kwargs['seasons'] = competition.seasons.extra(
                    select={'current': "id = %s"},
                    select_params=(season.pk,))

                # List of venues setup for this season.
                kwargs['venues'] = season.venues.all()

                if division_slug:
                    division = get_object_or_404(season.divisions,
                                                 slug=division_slug,
                                                 draft=False)
                    kwargs['division'] = division

                    # Because Division and StageGroup use the same template
                    # we will use special context value of parent as the base
                    # for ladders and matches_by_date (both models have these
                    # methods implemented.
                    kwargs['parent'] = division

                    # So we can build a navigation hierarchy, select extra
                    # column which tells us if the division in a list of
                    # divisions is the current scope.
                    kwargs['divisions'] = season.divisions.extra(
                        select={'current': "id = %s"},
                        select_params=(division.pk,))

                    if stage_slug:
                        stage = get_object_or_404(
                            division.stages.annotate(
                                pool_count=Count('pools')),
                            slug=stage_slug)
                        kwargs['stage'] = stage

                        if pool_slug:
                            pool = get_object_or_404(stage.pools,
                                                     slug=pool_slug)
                            kwargs['pool'] = pool

                            # Overload the parent set above.
                            kwargs['parent'] = pool

                    if team_slug:
                        team = get_object_or_404(division.teams.select_related('club'),
                                                 slug=team_slug)
                        kwargs['team'] = team

                        # List of players for this team.
                        players = team.people.select_related('person').filter(is_player=True)
                        players = players.extra(
                            select={'has_number': 'number IS NULL'},
                            order_by=('has_number', 'number',
                                      'person__last_name',
                                      'person__first_name'))
                        kwargs['players'] = players

                    if match_pk:
                        match = get_object_or_404(division.matches,
                                                  pk=match_pk)
                        kwargs['match'] = match

                elif match_pk:
                    match = get_object_or_404(season.matches, pk=match_pk)
                    kwargs['match'] = match

        if club_slug:
            club = get_object_or_404(Club, slug=club_slug)
            kwargs['club'] = club
            season = kwargs.get('season')
            if season:
                kwargs['teams'] = club.teams.filter(division__season=season) \
                                            .prefetch_related('division',
                                                              'stage_group')

        logger.debug('o_node = {0}'.format(o_node))

        logger.debug('kwargs["node"] = {node}'.format(node=o_node))

        extra_context = kwargs.pop('extra_context', {})
        extra_context.update(kwargs)
        kwargs['extra_context'] = extra_context

        return f(*args, **kwargs)
    return _decorated
competition_slug = method_decorator(competition_by_slug)



import logging
import os.path

from django.core.files.base import ContentFile
from django.core.files.storage import FileSystemStorage as FileSystemStorageBase
from django.db.models.fields.files import ImageFieldFile
from django.utils.six.moves.urllib.parse import urljoin
from PIL import Image
from touchtechnology.common.default_settings import STORAGE_FOLDER, STORAGE_URL

try:
    from tenant_schemas.storage import TenantStorageMixin
except ImportError:
    TenantStorageMixin = type('TenantStorageMixin', (), {})


logger = logging.getLogger(__name__)

__all__ = (
    'FileSystemStorage',
    'MakeDirectoryMixin',
    'OverwriteMixin',
    'ResizedImageFieldFile',
    'WalkMixin',
)


class MakeDirectoryMixin(object):

    def makedirs(self, name, path=None):
        parts = list(filter(None, (self.location, path, name)))
        directory = os.path.join(*parts)
        os.makedirs(directory)
        return os.path.join(*parts[1:])

    def mkdir(self, name, path=None):
        directory = self.path(os.path.join(
            *list(filter(None, (path, name)))))
        os.mkdir(directory)
        if path is None:
            return name
        return os.path.join(path, name)

    def rmdir(self, path):
        directory = self.path(path)
        os.rmdir(directory)


class OverwriteMixin(object):

    def get_available_name(self, name):
        while self.exists(name):
            self.delete(name)
        return name


class WalkMixin(object):

    def walk(self, top=None):
        """
        Generator that returns tuples similar to the built-in `os.walk`, but
        all paths are calculated from the Storage API `url` method.
        """
        if top is None:
            top = '.'

        # determine the directories and files in the present location, and
        # calculate their full path relative to the "top".
        directories, files = self.listdir(top)
        directories.sort()
        files.sort()
        files = [os.path.join(top, name) for name in files]

        # fetch our "url" with respect to this Storage class.
        yield (
            self.url(top),
            zip([self.url(d) for d in directories], directories),
            zip([self.url(f) for f in files], files),
        )

        # for any child directories, recursively descend and repeat.
        for d in directories:
            for triple in self.walk(os.path.join(top, d)):
                yield triple


class FileSystemStorage(MakeDirectoryMixin, WalkMixin, TenantStorageMixin,
                        FileSystemStorageBase):

    def __init__(self, location=STORAGE_FOLDER, base_url=STORAGE_URL,
                 *args, **kwargs):
        super(FileSystemStorage, self).__init__(*args, **kwargs)
        if location is not None:
            self.location = os.path.join(self.location, location)
        if base_url is not None:
            self.base_url = urljoin(self.base_url, base_url)

    def writable(self, path):
        if not self.exists(self.path(path)):
            raise OSError(2, os.strerror(2), path)
        return os.access(self.path(path), os.W_OK)


class ResizedImageFieldFile(ImageFieldFile):

    def __init__(self, instance, field_name, size, background):
        self.field_name = field_name
        self.maxwidth, self.maxheight = size
        self.background = background
        field = instance._meta.get_field(field_name)
        path, filename = os.path.split(getattr(instance, field_name).name)
        dim = '%sx%s' % (self.maxwidth, self.maxheight)
        name = os.path.join(path, dim, '%s.jpg' % instance.pk)
        super(ResizedImageFieldFile, self).__init__(instance, field, name)

    def regenerate(self):
        image_field = getattr(self.instance, self.field_name)
        if not self.storage.exists(image_field):
            logger.warning('Original file "{0}" does not exist, failed to '
                           'regenerate sized image.'.format(image_field))
            return

        path = os.path.dirname(self.name)
        if not self.storage.exists(path):
            self.storage.makedirs(path)

        original = self.storage.open(image_field)
        image = Image.open(original)
        if image.mode != 'RGB':
            image = image.convert('RGB')

        # determine the dimensions of the original image
        (__, __, w, h) = image.getbbox()

        # determine the new width & height for the image dimensions
        adjusted_width = int(w * (float(self.maxheight) / float(h)))
        adjusted_height = int(h * (float(self.maxwidth) / float(w)))

        # scale the image to the new size
        x_scale = min(self.maxwidth, adjusted_width)
        y_scale = min(self.maxheight, adjusted_height)
        small = image.resize((x_scale, y_scale), Image.ANTIALIAS)

        # create a new image object onto which we will paste the
        # resized original
        dim = (self.maxwidth, self.maxheight)
        thumbnail = Image.new("RGB", dim, self.background)

        # paste the resized image onto the thumbnail canvas
        x_offset = self.maxwidth - x_scale // 2
        y_offset = max(0, self.maxheight - y_scale // 2)
        offset = (x_offset, y_offset)
        thumbnail.paste(small, offset)

        f = ContentFile(thumbnail.tostring("jpeg", "RGB"))
        self.storage.save(self.name, f)



from django.conf.urls import include, url
from example_app.models import Relative, TestDateTimeField
from touchtechnology.common.sites import Application


class TestDateTimeFieldSite(Application):

    def __init__(self, name='datetime', app_name='datetime', *args, **kwargs):
        super(TestDateTimeFieldSite, self).__init__(
            name=name, app_name=app_name, *args, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url('^$', self.index, name='index'),
        ]
        return urlpatterns

    def index(self, request, *args, **kwargs):
        return self.generic_edit_related(
            request, TestDateTimeField, Relative)


class TestQueryStringSite(Application):

    def __init__(self, name='querystring', app_name='querystring', *args,
                 **kwargs):
        super(TestQueryStringSite, self).__init__(
            name=name, app_name=app_name, *args, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url(r'^$', self.index, name='index'),
        ]
        return urlpatterns

    def index(self, request, *args, **kwargs):
        queryset = TestDateTimeField.objects.order_by('pk')

        # naive filtering without a form - nasty, i know...
        year = request.GET.get('year')
        if year:
            queryset = queryset.filter(datetime__year=year)
            month = request.GET.get('month')
            if month:
                queryset = queryset.filter(datetime__month=month)

        return self.generic_list(request, TestDateTimeField,
                                 queryset=queryset)


class TestContextProcessorsSite(Application):

    def __init__(self, name='context', app_name='context', *args, **kwargs):
        super(TestContextProcessorsSite, self).__init__(
            name=name, app_name=app_name, *args, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url(r'^env$', self.env, name='env'),
            url(r'^site$', self.site_view, name='site'),
            url(r'^tz$', self.tz, name='tz'),
        ]
        return urlpatterns

    def env(self, request, *args, **kwargs):
        return self.render(request, self.template_path('env.html'), {})

    def site_view(self, request, *args, **kwargs):
        return self.render(request, self.template_path('site.html'), {})

    def tz(self, request, *args, **kwargs):
        return self.render(request, self.template_path('tz.html'), {})


class TestPaginationSite(Application):

    def __init__(self, name='pagination', app_name='pagination',
                 *args, **kwargs):
        super(TestPaginationSite, self).__init__(
            name=name, app_name=app_name, *args, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url(r'^setting-only/$',
                self.setting_only, name='setting_only'),
            url(r'^parameter/(?P<paginate_by>\d+)/$',
                self.parameter, name='parameter'),
        ]
        return urlpatterns

    def setting_only(self, request, *args, **kwargs):
        return self.generic_list(request, TestDateTimeField)

    def parameter(self, request, paginate_by, *args, **kwargs):
        return self.generic_list(request, TestDateTimeField,
                                 paginate_by=paginate_by)


class TestGenericViewsSite(Application):

    def __init__(self, name='generic', app_name='generic', *args, **kwargs):
        super(TestGenericViewsSite, self).__init__(
            name=name, app_name=app_name, *args, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url(r'^vanilla/', include(([
                url(r'^$', self.list, name='list'),
                url(r'^add/$', self.edit, name='add'),
                url(r'^(?P<pk>\d+)/$', self.detail, name='detail'),
                url(r'^(?P<pk>\d+)/edit/$', self.edit, name='edit'),
                url(r'^(?P<pk>\d+)/delete/$', self.delete, name='delete'),
                url(r'^(?P<pk>\d+)/perms/$', self.perms, name='perms'),
                url(r'^edit/$', self.edit_multiple, name='edit'),
            ], self.app_name), namespace='vanilla')),
            url(r'^permissions/', include(([
                url(r'^$', self.list_with_perms, name='list'),
                url(r'^add/$', self.edit_with_perms, name='add'),
                url(r'^(?P<pk>\d+)/$', self.detail_with_perms, name='detail'),
                url(r'^(?P<pk>\d+)/edit/$', self.edit_with_perms, name='edit'),
                url(r'^(?P<pk>\d+)/delete/$',
                    self.delete_with_perms, name='delete'),
                url(r'^edit/$', self.edit_multiple_with_perms, name='edit'),
            ], self.app_name), namespace='permissions')),
        ]
        return urlpatterns

    def list(self, request, **extra_context):
        return self.generic_list(request, TestDateTimeField,
                                 extra_context=extra_context)

    def detail(self, request, pk, **extra_context):
        return self.generic_detail(request, TestDateTimeField, pk=pk,
                                   extra_context=extra_context)

    def edit(self, request, pk=None, **extra_context):
        return self.generic_edit(request, TestDateTimeField, pk=pk,
                                 extra_context=extra_context)

    def delete(self, request, pk, **kwargs):
        return self.generic_delete(request, TestDateTimeField, pk=pk, **kwargs)

    def perms(self, request, pk, **extra_context):
        return self.generic_permissions(request, TestDateTimeField, pk=pk,
                                        extra_context=extra_context)

    def edit_multiple(self, request, **extra_context):
        return self.generic_edit_multiple(request, TestDateTimeField,
                                          extra_context=extra_context)

    def list_with_perms(self, request, **extra_context):
        return self.generic_list(request, TestDateTimeField,
                                 permission_required=True,
                                 extra_context=extra_context)

    def edit_with_perms(self, request, pk=None, **extra_context):
        return self.generic_edit(request, TestDateTimeField, pk=pk,
                                 permission_required=True,
                                 extra_context=extra_context)

    def delete_with_perms(self, request, pk, **kwargs):
        return self.generic_delete(request, TestDateTimeField, pk=pk,
                                   permission_required=True, **kwargs)

    def edit_multiple_with_perms(self, request, **extra_context):
        return self.generic_edit_multiple(request, TestDateTimeField,
                                          permission_required=True,
                                          extra_context=extra_context)

    def detail_with_perms(self, request, pk, **extra_context):
        return self.generic_detail(request, TestDateTimeField, pk=pk,
                                   # permission_required=True,
                                   extra_context=extra_context)



from __future__ import division

import decimal
import operator

from pyparsing import (
    Forward, Literal, Optional, StringEnd, Word, ZeroOrMore, alphanums, alphas,
    nums,
)

__all__ = ('Calculator', 'BonusPointCalculator')

Zero = decimal.Decimal(0)

# Atoms

Identifier = Word(alphas, alphanums + '_').setName('identifier')
Integer = (Optional('-') + Word(nums)).setParseAction(
    lambda s, loc, toks: ''.join(toks)).setName('integer')

# Literals

LeftParenthesis = Literal('(')
RightParenthesis = Literal(')')

LeftBracket = Literal('[')
RightBracket = Literal(']')

Colon = Literal(':')
Comma = Literal(',')

# Groups of Literals (all OR cases)
# Note: order matters, longer tokens must be listed first!
#   eg. '>=' before '>'

Addition = (Literal('+') | Literal('-')).setName('addop')
Multiplication = (Literal('*') | Literal('/')).setName('mulop')
Comparison = (
    Literal('=') | Literal('>=') | Literal('<=') | Literal('>') | Literal('<')
).setName('comparison')


class Calculator(object):

    def __init__(self, instance, *args, **kwargs):
        Expression = Forward()
        Atom = (
            (Identifier | Integer).setParseAction(self._push) |
            (LeftParenthesis + Expression.suppress() + RightParenthesis)
        ).setName('atom')
        Terminal = Atom + ZeroOrMore((Multiplication + Atom)
                                     .setParseAction(self._push))
        Expression << Terminal + ZeroOrMore((Addition + Terminal)
                                            .setParseAction(self._push))

        self.Operators = {
            "+": operator.add,
            "-": operator.sub,
            "*": operator.mul,
            "/": operator.truediv,
        }

        self.instance = instance
        self.pattern = Expression + StringEnd()
        self.stack = []

    def evaluate(self):
        try:
            token = self.stack.pop()
        except IndexError:
            return None
        if token in self.Operators:
            func = self.Operators.get(token)
            rhs = self.evaluate()
            lhs = self.evaluate()
            return func(lhs, rhs)
        elif Identifier.re.match(token):
            return getattr(self.instance, token, Zero)
        try:
            return decimal.Decimal(token)
        except decimal.InvalidOperation:
            return Zero

    def parse(self, data):
        self.pattern.parseString(data)

    def _push(self, string, position, tokens):
        """Add an element to the parse stack"""
        self.stack.append(tokens[0])


class BonusPointCalculator(Calculator):
    """
    Examples:

        Sydney Uni Touch
        "[win=1, score_against=0, forfeit_for=0: 1] + [loss=1, margin<=2: 1]"

        Super 14 Rugby
        "[tries>=4: 1] + [loss=1, margin<=7: 1]"
    """
    def __init__(self, *args, **kwargs):
        super(BonusPointCalculator, self).__init__(*args, **kwargs)

        self.Operators.update({
            "=": operator.eq,
            ">": operator.gt,
            "<": operator.lt,
            ">=": operator.ge,
            "<=": operator.le,
            ",": operator.and_,
            "[": lambda a, b: a and b or Zero,
            "]": lambda a, b: a and b or Zero,
        })

        BonusPointAddition = Addition | Comparison

        Expression = Forward()
        Atom = (
            (Identifier | Integer).setParseAction(self._push) |
            (LeftParenthesis + Expression.suppress() + RightParenthesis)
        )
        Terminal = Atom + ZeroOrMore((Multiplication + Atom)
                                     .setParseAction(self._push))
        Expression << Terminal + ZeroOrMore((BonusPointAddition + Terminal)
                                            .setParseAction(self._push))
        Condition = Expression + ZeroOrMore((Comma + Expression)
                                            .setParseAction(self._push))

        Rule = (LeftBracket +
                Condition +
                Colon +
                Expression +
                RightBracket).setParseAction(self._push)
        Chain = Rule + ZeroOrMore((Addition + Rule).setParseAction(self._push))

        self.pattern = Chain + StringEnd()

    def parse(self, data):
        if data.strip():
            self.pattern.parseString(data)



import logging
import subprocess

import requests
from django.conf import settings
from django.utils.html import strip_spaces_between_tags

SERVER = getattr(settings, 'PRINCE_SERVER', None)
BINARY = getattr(settings, 'PRINCE_BINARY', '/usr/local/bin/prince')
BASE_URL = getattr(settings, 'PRINCE_BASE_URL', None)

logger = logging.getLogger(__name__)


def prince(html, base_url=BASE_URL, ttl=300, **kwargs):
    # When celery and django-tenant-schemas are involved, this get's a bit
    # weird. This has bitten once in production so lets log it and see else
    # might cause grief.
    for kw, arg in kwargs.items():
        logger.warn('Unexpected keyword argument: %s=%r', kw, arg)

    logger.debug('base_url: %s', base_url)
    logger.debug('ttl: %s', ttl)

    logger.debug('content-length: %s', len(html))
    html = strip_spaces_between_tags(html)
    logger.debug('content-length-stripped: %s', len(html))

    if SERVER:
        logger.debug('server: %s', SERVER)
        if isinstance(html, unicode):
            html = html.encode('utf8')

        headers = {}
        if base_url:
            headers['base_url'] = base_url
        for key, value in headers.items():
            logger.debug('header: %s=%s', key, value)

        res = requests.post(
            "http://%s/" % settings.PRINCE_SERVER,
            data=html,
            headers=headers)
        res.raise_for_status()

        out = res.content
        logger.info('Remote PDF generation via %s complete.', SERVER)

    else:
        command = [BINARY, '--input=html']
        if base_url:
            command += ['--baseurl=%s' % base_url]
        command += ['-']
        logger.debug('command: %s', ' '.join(command))

        p = subprocess.Popen(
            command,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        out, err = p.communicate(html.encode('utf8'))
        logger.info('Local PDF generation complete.')

        if err:
            logger.error(err)

    return out



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Chunk',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('copy', touchtechnology.common.db.models.HTMLField(verbose_name='Page Copy', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Content',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('copy', touchtechnology.common.db.models.HTMLField(blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='NodeContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('copy', touchtechnology.common.db.models.HTMLField(verbose_name='Page Copy', blank=True)),
                ('node', models.ForeignKey(related_name=b'contents', verbose_name='Node', to='common.SitemapNode', on_delete=models.PROTECT)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('template', touchtechnology.common.db.models.TemplatePathField(max_length=200, verbose_name='Template', blank=True)),
                ('keywords', models.CharField(help_text='This should be a comma-separated list of terms that indicate the content of this page - used to assist Search Engines rank your page.', max_length=255, verbose_name='Keywords', blank=True)),
                ('description', models.CharField(help_text='Search Engines may use this when determining the relevance of your page.', max_length=255, verbose_name='Description', blank=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PageContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.SlugField(default=b'copy', max_length=20, verbose_name='CSS class', choices=[(b'copy', b'copy')])),
                ('sequence', models.PositiveIntegerField(verbose_name='Sequence')),
                ('copy', touchtechnology.common.db.models.HTMLField(verbose_name='Copy', blank=True)),
                ('page', models.ForeignKey(related_name=b'content', verbose_name='Page', to='content.Page', on_delete=models.PROTECT)),
            ],
            options={
                'ordering': ('sequence',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Placeholder',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('path', models.CharField(max_length=255, verbose_name='Module path')),
                ('namespace', models.CharField(max_length=255, verbose_name='Namespace', db_index=True)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PlaceholderKeywordArgument',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(max_length=200, verbose_name='Key')),
                ('value', models.TextField(default='null', help_text='Enter a valid JSON object', verbose_name='Value', blank=True)),
                ('node', models.ForeignKey(related_name=b'kw', verbose_name='Node', to='common.SitemapNode', on_delete=models.PROTECT)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Redirect',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('source_url', models.CharField(help_text='The path that will trigger the redirection.', max_length=250, verbose_name='Source URL')),
                ('destination_url', models.CharField(help_text='The URL or path that the browser will be sent to.', max_length=500, verbose_name='Destination URL')),
                ('label', models.CharField(max_length=100, verbose_name='Label', blank=True)),
                ('active', models.BooleanField(default=True, verbose_name='Active')),
                ('permanent', models.BooleanField(default=False, verbose_name='Permanent')),
            ],
            options={
                'ordering': ('destination_url',),
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='placeholderkeywordargument',
            unique_together=set([('node', 'key')]),
        ),
    ]



from __future__ import unicode_literals

from django.template import Context, Template
from django.test.utils import override_settings
from django.utils.http import urlencode
from django.utils.six.moves.urllib.parse import urlparse, urlunparse
from test_plus import TestCase

CSSIFY_TEMPLATE = Template("{% load common %}{{ value|cssify }}")
TWITTIFY_TEMPLATE = Template("{% load common %}{{ value|twittify }}")


class CssifyTest(TestCase):

    def test_cssify_str(self):
        context = Context({'value': 'some-normal-slug'})
        value = CSSIFY_TEMPLATE.render(context)
        self.assertEqual('some_normal_slug', value)

    def test_cssify_unicode(self):
        context = Context({'value': 'some-normal-slug'})
        value = CSSIFY_TEMPLATE.render(context)
        self.assertEqual('some_normal_slug', value)

    def test_cssify_none(self):
        context = Context({'value': None})
        value = CSSIFY_TEMPLATE.render(context)
        self.assertEqual('', value)


class TwittifyTest(TestCase):

    def test_twittify(self):
        context = Context({'value': "@goodtune"})
        value = TWITTIFY_TEMPLATE.render(context)
        expected = """
        @<a class="twitter user" target="_blank"
            href="http://twitter.com/goodtune">goodtune</a>
        """
        self.assertHTMLEqual(expected, value)

    def test_twittify_invalid(self):
        context = Context({'value': "goodtune"})
        value = TWITTIFY_TEMPLATE.render(context)
        expected = "goodtune"
        self.assertHTMLEqual(expected, value)


@override_settings(ROOT_URLCONF='example_app.urls')
class QueryStringTest(TestCase):

    fixtures = ['query_string']

    def setUp(self):
        self.query = dict(year=2013)
        self.url = urlparse(self.reverse('querystring:index'))

    def test_no_filter_no_page(self):
        self.get('querystring:index')

        self.assertResponseContains('<a href="?page=3">3</a>')
        self.assertResponseNotContains('<a href="?page=4">4</a>')

        self.assertResponseContains('<span id="5">5</span>')
        self.assertResponseNotContains('<span id="9">9</span>')

    def test_no_filter_page(self):
        query = dict(page=2)
        url = self.url._replace(query=urlencode(query, True))
        self.get(urlunparse(url))

        self.assertResponseContains('<a href="?page=3">3</a>')
        self.assertResponseNotContains('<a href="?page=4">4</a>')

        self.assertResponseNotContains('<span id="5">5</span>')
        self.assertResponseContains('<span id="9">9</span>')

    def test_filter_no_page(self):
        url = self.url._replace(query=urlencode(self.query, True))
        self.get(urlunparse(url))

        link = '<a href="?year={year}&amp;page={page}">{page}</a>'
        self.assertResponseContains(link.format(page=2, **self.query))
        self.assertResponseNotContains(link.format(page=3, **self.query))

        self.assertResponseContains('<span id="5">5</span>')
        self.assertResponseNotContains('<span id="9">9</span>')

    def test_filter_page(self):
        query = dict(page=2, **self.query)
        url = self.url._replace(query=urlencode(query, True))
        self.get(urlunparse(url))

        link = '<a href="?year={year}&amp;page={page}">{page}</a>'
        self.assertResponseContains(link.format(page=2, **self.query))
        self.assertResponseNotContains(link.format(page=3, **self.query))

        self.assertResponseNotContains('<span id="5">5</span>')
        self.assertResponseContains('<span id="9">9</span>')


@override_settings(ROOT_URLCONF='example_app.urls')
class ContextTest(TestCase):

    def test_env(self):
        self.get('context:env')
        self.assertResponseContains('dev', html=False)

    def test_tz(self):
        self.get('context:tz')
        self.assertResponseContains('UTC', html=False)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('competition', '0015_person_user_relation'),
    ]

    operations = [
        migrations.AddField(
            model_name='season',
            name='forfeit_notifications',
            field=models.ManyToManyField(help_text='When a team advises they are forfeiting, notify the opposition team plus these people.', to=settings.AUTH_USER_MODEL, blank=True),
        ),
    ]



from django.conf import settings
from django.db.models import (
    Case, DecimalField, ExpressionWrapper, F, Func, When,
)
from django.db.models.query import QuerySet
from django.utils import timezone
from django.utils.module_loading import import_string


class DivisionQuerySet(QuerySet):
    """
    QuerySet to allow divisions to be easily filtered by their draft status.
    """
    def public(self):
        return self.filter(draft=False)


class StageQuerySet(QuerySet):

    def with_ladder(self):
        return self.filter(
            keep_ladder=True, ladder_summary__isnull=False).distinct()


class MatchQuerySet(QuerySet):

    def future(self, date=None):
        if date is None:
            date = timezone.now().date()
        return self.filter(date__gte=date)

    def playable(self):
        return self.exclude(is_bye=True)

    def videos(self):
        return self.filter(videos__isnull=False).order_by('datetime').distinct()

    def _rank_importance(self):
        """
        Find the rank_importance of a match based on the competition hierarchy.
        """
        return self.annotate(
            importance=Case(
                When(
                    rank_importance__isnull=False,
                    then=F('rank_importance')),
                When(
                    stage_group__rank_importance__isnull=False,
                    then=F('stage_group__rank_importance')),
                When(
                    stage__rank_importance__isnull=False,
                    then=F('stage__rank_importance')),
                When(
                    stage__division__rank_importance__isnull=False,
                    then=F('stage__division__rank_importance')),
                When(
                    stage__division__season__rank_importance__isnull=False,
                    then=F('stage__division__season__rank_importance')),
                When(
                    stage__division__season__competition__rank_importance__isnull=False,
                    then=F('stage__division__season__competition__rank_importance')),
            ),
        )


class LadderEntryQuerySet(QuerySet):

    def _all(self):
        qs = self.annotate(
            diff=F('score_for') - F('score_against'),
            margin=Func(F('score_for') - F('score_against'), function='ABS'),
        )

        qs = qs.select_related('team__club')

        qs = qs.annotate(
            division=Case(
                When(
                    team__rank_division__isnull=False,
                    then=F('team__rank_division')),
                When(
                    team__division__rank_division__isnull=False,
                    then=F('team__division__rank_division')),
            ),
        )

        qs = qs.annotate(
            importance=Case(
                When(
                    match__rank_importance__isnull=False,
                    then=F('match__rank_importance')),
                When(
                    match__stage_group__rank_importance__isnull=False,
                    then=F('match__stage_group__rank_importance')),
                When(
                    match__stage__rank_importance__isnull=False,
                    then=F('match__stage__rank_importance')),
                When(
                    match__stage__division__rank_importance__isnull=False,
                    then=F('match__stage__division__rank_importance')),
                When(
                    match__stage__division__season__rank_importance__isnull=False,
                    then=F('match__stage__division__season__rank_importance')),
                When(
                    match__stage__division__season__competition__rank_importance__isnull=False,
                    then=F(
                        'match__stage__division__season__competition__rank_importance')),
            ),
        )

        RANK_POINTS_FUNC = getattr(
            settings, 'TOURNAMENTCONTROL_RANK_POINTS_FUNC',
            'tournamentcontrol.competition.rank.points_func'
        )
        rank_points = import_string(RANK_POINTS_FUNC)
        qs = qs.annotate(
            rank_points=ExpressionWrapper(
                rank_points() * F('importance'),
                output_field=DecimalField(),
            ),
        )

        return qs


class StatisticQuerySet(QuerySet):

    def played(self):
        return self.exclude(played=0)
